package it.cavallium.filequeue;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.nio.file.Path;

public final class QueueToConsumerMemorySegment<T> implements IQueueToConsumer<T> {

	private final SimpleQueueMemorySegment<T> queueMemorySegment;
	private final QueueToConsumer<T> queue;
	private final Path dir;

	public QueueToConsumerMemorySegment(Path file,
										String dbName,
										Serializer<T> serializer,
										Deserializer<T> deserializer,
										QueueConsumer<T> consumer,
										long segmentSize) {
		try {
			this.dir = file.resolve(dbName);
			this.queueMemorySegment = new SimpleQueueMemorySegment<>(dir,
					dbName,
					serializer,
					deserializer,
					segmentSize
			);
		} catch (IOException e) {
			throw new UncheckedIOException(e);
		}
		this.queue = new QueueToConsumer<>(queueMemorySegment, consumer) {
			@Override
			protected String getQueueManagerName() {
				return super.getQueueManagerName() + "-" + dbName;
			}
		};
	}

	@Override
	public void add(T value) {
		queue.add(value);
	}

	@Override
	public void close() {
		queue.close();
		queueMemorySegment.close();
		try {
			FileUtils.deleteDirectory(dir);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void startQueue() {
		queue.startQueue();
	}

	@Override
	public void runStep(boolean park) {
		queue.runStep(park);
	}
}
