package it.cavallium.filequeue;

import java.util.concurrent.ConcurrentLinkedDeque;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

public abstract class TestQueueToConsumer {

	public abstract SimpleQueue<String> getSimpleQueue();

	@Test
	public void test() throws Exception {
		var q = getSimpleQueue();
		AtomicBoolean ab = new AtomicBoolean();
		try (var qtc = new QueueToConsumer<>(q, value -> {
			if (ab.get()) {
				ab.set(false);
				return false;
			} else {
				ab.set(true);
				return true;
			}
		})) {
			qtc.startQueue();
			qtc.add("ciao");
			qtc.add("mondo");
			for (int i = 0; i < 1000; i++) {
				qtc.add(i + "n");
			}
			Thread.sleep(100);
		} catch (InterruptedException e) {
			throw new RuntimeException(e);
		}
		if (q instanceof AutoCloseable autoCloseable) {
			autoCloseable.close();
		}
	}

	@Test
	public void testConcurrency() throws Exception {
		var q = getSimpleQueue();
		AtomicBoolean ab = new AtomicBoolean();
		CountDownLatch cdl = new CountDownLatch(1);
		var referenceQueue = IntStream
				.range(0, 1000)
				.mapToObj(Integer::toString)
				.collect(Collectors.toCollection(ConcurrentLinkedDeque::new));
		try (var qtc = new QueueToConsumer<>(q, value -> {
			if (ab.get()) {
				ab.set(false);
				return false;
			} else {
				ab.set(true);
				Assertions.assertEquals(referenceQueue.pop(), value);
				if (referenceQueue.isEmpty()) {
					cdl.countDown();
				}
				return true;
			}
		})) {
			var t = new Thread(() -> {
				while (!Thread.interrupted()) {
					qtc.runStep(false);
				}
			});
			t.setDaemon(true);
			t.setName("step");
			t.start();
			for (String s : referenceQueue.toArray(String[]::new)) {
				qtc.add(s);
			}
			cdl.await();
			t.interrupt();
			t.join();
			Assertions.assertEquals(0, referenceQueue.size());
		} catch (InterruptedException e) {
			throw new RuntimeException(e);
		}
		if (q instanceof AutoCloseable autoCloseable) {
			autoCloseable.close();
		}
	}
}
