package it.cavallium.filequeue;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Comparator;

public class TestQueueToConsumerFileChannel extends TestQueueToConsumer {

	private static final long SEGMENT_SIZE = 24;
	private Path dir;

	@BeforeEach
	public void setup() throws IOException {
		dir = Files.createTempDirectory("queue");
	}

	@AfterEach
	public void tearDown() throws IOException {
		Path pathToBeDeleted = dir;
		try (var fs = Files.walk(pathToBeDeleted)) {
			fs.sorted(Comparator.reverseOrder())
					.forEach(f -> {
						try {
							Files.deleteIfExists(f);
						} catch (IOException e) {
							throw new RuntimeException(e);
						}
					});
		}
	}

	@Override
	public SimpleQueue<String> getSimpleQueue() {
		try {
			return new SimpleQueueMemorySegment<>(dir, "test", new Serializer<>() {
				@Override
				public byte[] serialize(String data) {
					return data.getBytes(StandardCharsets.UTF_8);
				}
			}, new Deserializer<>() {
				@Override
				public String deserialize(byte[] data) {
					return new String(data, StandardCharsets.UTF_8);
				}
			}, SEGMENT_SIZE);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
