package it.cavallium.filequeue;

import java.util.ArrayDeque;
import java.util.Collection;
import java.util.Iterator;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedDeque;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

public class TestQueueToConsumerJava extends TestQueueToConsumer {

	@Override
	public SimpleQueue<String> getSimpleQueue() {
		return new SimpleQueueJava<>(new ConcurrentLinkedDeque<>());
	}

	@Test
	public void testNoQueue() {
		var q = new SimpleQueueJava<>(new Queue<String>() {
			@Override
			public boolean add(String s) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean offer(String s) {
				Assertions.fail();
				return false;
			}

			@Override
			public String remove() {
				Assertions.fail();
				return null;
			}

			@Override
			public String poll() {
				return null;
			}

			@Override
			public String element() {
				Assertions.fail();
				return null;
			}

			@Override
			public String peek() {
				return null;
			}

			@Override
			public int size() {
				return 0;
			}

			@Override
			public boolean isEmpty() {
				return true;
			}

			@Override
			public boolean contains(Object o) {
				Assertions.fail();
				return false;
			}

			@Override
			public Iterator<String> iterator() {
				Assertions.fail();
				return null;
			}

			@Override
			public Object[] toArray() {
				Assertions.fail();
				return new Object[0];
			}

			@Override
			public <T> T[] toArray(T[] a) {
				Assertions.fail();
				return null;
			}

			@Override
			public boolean remove(Object o) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean containsAll(Collection<?> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean addAll(Collection<? extends String> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean removeAll(Collection<?> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean retainAll(Collection<?> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public void clear() {
			}
		});
		AtomicReference<String> lastConsumed = new AtomicReference<>();
		try (var consumer = new QueueToConsumer<>(q, value -> {
			lastConsumed.set(value);
			return true;
		})) {
			for (int i = 0; i < 10_000; i++) {
				var s = Integer.toString(i);
				consumer.add(s);
				Assertions.assertEquals(s, lastConsumed.get());
			}
		}
	}

	@Test
	public void testSmallQueueThenNone() {
		var maxQueueSize = 10;
		var shouldReject = new AtomicBoolean(true);
		var q = new SimpleQueueJava<>(new Queue<String>() {
			final Queue<String> smallQueue = new ArrayDeque<>(maxQueueSize);
			@Override
			public boolean add(String s) {
				var added = smallQueue.add(s);
				Assertions.assertTrue(smallQueue.size() <= maxQueueSize);
				return added;
			}

			@Override
			public boolean offer(String s) {
				var added = smallQueue.offer(s);
				Assertions.assertTrue(smallQueue.size() <= maxQueueSize);
				return added;
			}

			@Override
			public String remove() {
				var removed = smallQueue.remove();
				Assertions.assertTrue(smallQueue.size() <= maxQueueSize);
				return removed;
			}

			@Override
			public String poll() {
				var removed = smallQueue.poll();
				Assertions.assertTrue(smallQueue.size() <= maxQueueSize);
				return removed;
			}

			@Override
			public String element() {
				var element = smallQueue.element();
				Assertions.assertTrue(smallQueue.size() <= maxQueueSize);
				return element;
			}

			@Override
			public String peek() {
				var element = smallQueue.peek();
				Assertions.assertTrue(smallQueue.size() <= maxQueueSize);
				return element;
			}

			@Override
			public int size() {
				return smallQueue.size();
			}

			@Override
			public boolean isEmpty() {
				return smallQueue.isEmpty();
			}

			@Override
			public boolean contains(Object o) {
				Assertions.fail();
				return false;
			}

			@Override
			public Iterator<String> iterator() {
				Assertions.fail();
				return null;
			}

			@Override
			public Object[] toArray() {
				Assertions.fail();
				return new Object[0];
			}

			@Override
			public <T> T[] toArray(T[] a) {
				Assertions.fail();
				return null;
			}

			@Override
			public boolean remove(Object o) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean containsAll(Collection<?> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean addAll(Collection<? extends String> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean removeAll(Collection<?> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public boolean retainAll(Collection<?> c) {
				Assertions.fail();
				return false;
			}

			@Override
			public void clear() {
				smallQueue.clear();
			}
		});
		AtomicReference<String> lastConsumed = new AtomicReference<>();
		try (var consumer = new QueueToConsumer<>(q, value -> {
			if (shouldReject.get()) {
				lastConsumed.set(null);
				return false;
			} else {
				lastConsumed.set(value);
				return true;
			}
		})) {
			for (int i = 0; i < maxQueueSize; i++) {
				var s = Integer.toString(i);
				consumer.add(s);
				Assertions.assertNull(lastConsumed.get());
			}
			shouldReject.set(false);
			for (int i = 0; i < maxQueueSize; i++) {
				var s = Integer.toString(i);
				consumer.runStep(false);
				Assertions.assertEquals(s, lastConsumed.get());
				Assertions.assertEquals(maxQueueSize - i - 1, q.size());
			}
			Assertions.assertEquals(0, q.size());
			for (int i = maxQueueSize; i < 10_000; i++) {
				var s = Integer.toString(i);
				consumer.add(s);

				Assertions.assertEquals(s, lastConsumed.get());
				Assertions.assertEquals(0, q.size());
			}
		}
	}
}
